# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Build tree module: use build tree user should provide the organism highly conserved proteins
or 16s SSU rRNA.
"""
import time

from physpetool.convert.concatenate import cocat_path
from physpetool.convert.fasta2phy import fasta2phy
from physpetool.phylotree.doclustalw import doclustalw_file, doclustalw
from physpetool.phylotree.dofasttree import doFastTree
from physpetool.phylotree.dogblocks import dogblocks
from physpetool.phylotree.doiqtree import doiqtree
from physpetool.phylotree.domafft import domafft_file, domafft
from physpetool.phylotree.domuscle import domuscle_file, domuscle
from physpetool.phylotree.doraxml import doraxml

# default arguments
from physpetool.phylotree.dotrimal import dotrimal
from physpetool.phylotree.log import getLogging

raxmlpara_pro = "-f a -m PROTGAMMAJTTX  -p 12345 -x 12345 -# 100 -n T1"
raxmlpara_dna = "-f a -m GTRGAMMA  -p 12345 -x 12345 -# 100 -n T1"
musclepara = "-maxiters 100"
gblockspara_pro = '-t=p -b5=h -e=-gb1'
gblockspara_dna = '-t=d -b5=h -e=-gb1'
clustalwpara = None
trimalpara = "-gt 1"
mafftpara = "--auto"

build_log = getLogging('Used time')

def build_hcp(in_put, out_put,
              args_muscle, args_muscle_p,
              args_clustalw, args_clustalw_p,
              args_mafft, args_mafft_p,
              args_gblocks, args_gblocks_p,
              args_trimal, args_trimal_p,
              args_raxml, args_raxml_p,
              args_fasttree, args_fasttree_p,
              args_iqtree,args_iqtree_p,
              args_thread):
    '''reconstruct phylogenetic tree by hcp method'''
    start = time.time()
    out_retrieve = in_put
    # set default aligned by muscle if not specify clustalw or mafft
    if args_clustalw:
        out_alg = doclustalw_file(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft_file(out_retrieve, out_put, args_mafft_p,args_thread)
    elif args_muscle:
        out_alg = domuscle_file(out_retrieve, out_put, args_muscle_p,args_thread)

    out_concat = cocat_path(out_alg)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_concat, args_trimal_p)
    elif args_gblocks:
        out_gblock = dogblocks(out_concat, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)

    # reconstruct tree
    if args_fasttree:
        doFastTree(out_f2p, out_put, args_fasttree_p, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end = time.time()
    build_log.info('Constructing species tree used time: {} Seconds'.format(end - start))


def build_srna(in_put, out_put,
               args_muscle, args_muscle_p,
               args_clustalw, args_clustalw_p,
               args_mafft, args_mafft_p,
               args_gblocks, args_gblocks_p,
               args_trimal, args_trimal_p,
               args_raxml, args_raxml_p,
               args_fasttree, args_fasttree_p,
               args_iqtree,args_iqtree_p,
               args_thread):
    '''reconstruct phylogenetic tree by ssu rna method'''
    start = time.time()
    out_retrieve = in_put
    # set default aligned by muscle if not specify clustalw or mafft
    if args_clustalw:
        out_alg = doclustalw(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft(out_retrieve, out_put, args_mafft_p)
    elif args_muscle:
        out_alg = domuscle(out_retrieve, out_put, args_muscle_p)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_alg, args_trimal_p)
    elif args_gblocks:
        if args_gblocks_p == gblockspara_pro:
            args_gblocks_p = gblockspara_dna
            out_gblock = dogblocks(out_alg, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)
    # reconstruct tree
    if args_fasttree:
        args_fasttree_p_add = "-nt " + args_fasttree_p.lstrip()
        doFastTree(out_f2p, out_put, args_fasttree_p_add, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        if args_raxml_p == raxmlpara_pro:
            args_raxml_p = raxmlpara_dna
            doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end = time.time()
    build_log.info('Constructing species tree used time: {} Seconds'.format(end - start))


# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################
"""
function to call FastTree and FastTree reconstruct tree.
"""
import os

import subprocess

from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath

logdofasttree = getLogging('FastTree')


def doFastTree(inputfile, outputfile, FastTreepara, thread):
    # Use FASTA format build tree
    input_fasta = inputfile.replace('.phy', '')
    FastTreePath = getlocalpath()
    thread_to_str = str(thread)
    out_tree_name = os.path.join(outputfile, "FastTree.tree")
    if not os.path.exists(outputfile):
        os.mkdir(outputfile)
    if thread_to_str is '1':
        cmd = FastTreePath + "/FastTree " + FastTreepara  +" "+input_fasta + " >" + out_tree_name
        subprocess.call(cmd, shell=True)
    else:
        # set the threads
        os.environ["OMP_NUM_THREADS"] = thread_to_str
        cmd = FastTreePath + "/FastTreeMP " + FastTreepara +" "+ input_fasta + " >" + out_tree_name
        subprocess.call(cmd, shell=True)
    logdofasttree.info("Phylogenetic species tree reconstructed by FastTree was completed")


if __name__ == '__main__':
    doFastTree("/home/yangfang/physpetools/testdata/temp/concatenate20160916155942/concatenate.fasta-gb1",
               "/home/yangfang/physpetools_data/test_fasttree/",
               "", "4"
               )



import os
proindexlist = [['ath:AT2G27530', 'zma:103638688', 'eco:b3984'], ['ath:ArthCp055', 'zma:845224', 'eco:b3295']]

db_dir ="/home/yangfang/PhySpeTree/revision/HCPDB/databasehcp/"
tem_dir = "/home/yangfang/PhySpeTree/revision/HCPDB/tem/"
if not os.path.isdir(tem_dir):
    os.mkdir(tem_dir)

p=1
for line in proindexlist:
    res = []
    for each in line:
       with open(db_dir+each + ".fasta") as f:
           tem = f.read().split("\n")
           tem[0] = tem[0].strip().split(":")[0]
           res.append("\n".join(tem) + "\n")
    w_path = open(tem_dir+ "p" + str(p) + ".fasta",'w')
    w_path.write("".join(res))



####deal with hcp db

db_dir = "/home/yangfang/PhySpeTree/revision/HCPDB/databasehcp_backup/"

w_dir = "/home/yangfang/PhySpeTree/revision/HCPDB/databasehcp/"

test =  []
with open("/home/yangfang/PhySpeTree/physpetools/physpetool/database/support_hcp_organism.txt") as f:
    for line in f:
        test.append(line.strip().split("\t")[0])



all_files = list(filter(lambda f: not f.startswith('.'), os.listdir(db_dir)))
s = set()
for line in all_files:
    abb_name = line.strip().split(":")[0]
    s.add(abb_name)

    # w_ = open(w_dir+abb_name+".fasta",'a')
    # seq_file = open(db_dir+line)
    # seq = seq_file.read()
    # w_.write(seq+"\n")
    # seq_file.close()
    # w_.close()





# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################


"""
Check input file is right.

"""

from physpetool.database.dbpath import getlocaldbpath
from physpetool.phylotree.log import getLogging
import os

from physpetool.utils.checkIsNum import is_number

dbpath = getlocaldbpath()
logchecking = getLogging('Checking organisms')


def check_organism(input, db_list):
    """
    check input organism
    :param input: a list contain species name
    :param db_list: a list file contain organism in corresponding database
    :return: inputlist: match in database mislist: can't match in database
    """
    originaList = input

    inputlist = []
    mislist = []
    spelist = os.path.join(dbpath, db_list)
    for line in originaList:
        abb = line.strip()
        for spename in open(spelist):
            spe_name_list = spename.strip().split('\t')
            spe_name = spe_name_list[0]
            if spe_name == abb:
                inputlist.append(abb)
                break
            else:
                pass
    # Get can't match list
    for che in inputlist:
        originaList.remove(che)
        mislist = originaList
    return inputlist, mislist


def check_organism_silva(input, db_list, recovery_list):
    """
    check input organism
    :param input: a list contain species name
    :param db_list: a list file contain organism in corresponding database
    :return: inputlist: match in database mislist: can't match in database
    """
    originaList = input
    re_re_list = []
    inputlist = []
    mislist = []
    spelist = os.path.join(dbpath, db_list)
    spe_name_db = set()
    for spename in open(spelist):
        spe_name_list = spename.strip().split('\t')
        spe_name = spe_name_list[0]
        spe_name_db.add(spe_name)

    for line in range(len(originaList)):
        abb = originaList[line].strip()
        if abb in spe_name_db:
            inputlist.append(abb)
            re_re_list.append(recovery_list[line])

    # Get can't match list
    for che in inputlist:
        originaList.remove(che)
        mislist = originaList
    return inputlist, mislist, re_re_list
def trans_abb_2_tax_for_surna(input):
    """
transform kegg abb name to ncbi taxonomy id for SURNA method
    :param input: input file contains kegg abb names
    :return:  the list contains taxonomy ids
    """
    originaList = []
    inputlist = []
    dic_trans_list = {}
    recovery_list = []
    for line in input:
        st = line.strip()
        originaList.append(st)
    originaList = removeEmptyStr(originaList)

    spelist = os.path.join(dbpath, "support_hcp_organism.txt")

    if not is_number(originaList[0]):
        with open(spelist) as f:
            for line in f:
                tem = line.strip().split('\t')
                dic_trans_list[tem[0]] = tem[1]
        for abb in originaList:
            if abb in dic_trans_list.keys():
                inputlist.append(dic_trans_list[abb])
                recovery_list.append(abb)
            else:
                logchecking.info(abb+"not in SSU rRNA DB and remove it reconstructed species tree.")
    else:
        inputlist = originaList
    return inputlist, recovery_list


def trans_tax_to_abb_for_hcp(input):
    """
transform ncbi taxonomy id to kegg abb name for HCP method
    :param input: input file contains ncbi taxonomy id
    :return: the list contains abb names
    """
    originaList = []
    inputlist = []
    dic_trans_list = {}
    recovery_dic = {}
    for line in input:
        st = line.strip()
        originaList.append(st)
    originaList = removeEmptyStr(originaList)
    spelist = os.path.join(dbpath, "support_hcp_organism.txt")

    if is_number(originaList[0]):
        with open(spelist) as f:
            for line in f:
                tem = line.strip().split('\t')
                dic_trans_list[tem[1]] = tem[0]
        for tax in originaList:
            inputlist.append(dic_trans_list[tax])
            recovery_dic[dic_trans_list[tax]] = tax
    else:
        inputlist = originaList

    return inputlist, recovery_dic

def checkKeggOrganism(input):
    """
Check input organisms list with KEGG database
    :param input: a open file list
    :return: a list contain organisms can be use construct phy tree
    """
    input_trans, recovery_dic = trans_tax_to_abb_for_hcp(input)
    inputlist, mislist = check_organism(input_trans, "support_hcp_organism.txt")
    if mislist:
        for misabb in mislist:
            logchecking.info("The species: {0} can't match in KEGG protein index database".format(misabb))
        logchecking.warning(
            "These species can't match in KEGG protein index database so removed and reconstruct phylogenetic tree.")
    return inputlist, recovery_dic


def checkSilvaOrganism(input):
    """
Check input organisms list with SILVA database
    :param input: a open file list
    :return: a list contain organisms can be use construct phy tree
    """
    input_trans,recovery_dic = trans_abb_2_tax_for_surna(input)
    inputlist, mislist,recovery_dic2 = check_organism_silva(input_trans, "support_srna_organism.txt",recovery_dic)
    if mislist:
        for misabb in mislist:
            logchecking.info("The organism: {0} can't match in SSU rRNA database".format(misabb))
        logchecking.warning(
            "These species can't match in SSU rRNA database so removing and reconstructing phylogenetic tree.")

    return inputlist, recovery_dic2


def removeEmptyStr(all):
    """
Remove empty string
    :param all: input a list
    :return: a new list
    """
    newList = []
    for var in all:
        if var:
            newList.append(var)
    return newList


def checkFile(filepath):
    """
check input files
    :param filepath: input files
    :return: input files
    """
    if os.path.isfile(filepath):
        return filepath
    else:
        print("Please input a exact file")
        exit(0)


def readIputFile(inputfile):
    """
    prepare and check input file list
    :param inputfile:
    :return: inputfile by check list
    """
    org_name = []
    with open(inputfile) as f:
        for name in f:
            each_name = name.strip()
            org_name.append(each_name)

    org_name_check = removeEmptyStr(org_name)
    return org_name_check

def recovery(file_path,re_dic):
    all_pre_file = list(filter(lambda f: not f.startswith('.'), os.listdir(file_path)))
    for i in all_pre_file:
        file_data = ''
        abs_file = os.path.join(file_path,i)
        with open(abs_file,"r") as f:
            for line in f:
                if line.startswith('>'):
                    name = line.strip()[1:]
                    line = line.replace(name, re_dic[name])
                file_data += line
        with open(abs_file,"w") as f2:
            f2.write(file_data)
def recovery_silva(file_path,re_dic,ssu_input):
    all_pre_file = list(filter(lambda f: not f.startswith('.'), os.listdir(file_path)))
    for i in all_pre_file:
        file_data = ''
        abs_file = os.path.join(file_path,i)
        idx =0
        with open(abs_file,"r") as f:
            for line in f:
                if line.startswith('>'):
                    name = line.strip()[1:]
                    line = line.replace(name, re_dic[idx])
                    idx +=1
                file_data += line
        with open(abs_file,"w") as f2:
            f2.write(file_data)


# coding=utf-8
# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################


"""
The PhySpeTree version and call software in PhySpeTree  version and citation.

"""
version = '0.3.9'

VERSION_DESC = (
    """
          -------------------------------------------------------------------------------------
                      PhySpeTree (%s) - Reconstruct Phylogenetic species Tree
          PhySpeTree document: https://yangfangs.github.io/physpetools/       
        
          Citation:
                  Fang, Y., Liu, C., Lin, J. et al. PhySpeTree: an automated pipeline for 
                  reconstructing phylogenetic species trees. BMC Evol Biol 19, 219 (2019) 
                  doi:10.1186/s12862-019-1541-x

          --------------------------------------------------------------------------------------
          """ % (version))


def version_infor():
    muscle_info = 'v3.8.31'
    clustalw2_info = '2.1'
    mafft_info = 'v7.429'
    trimal_info = '1.2rev59'
    gblocks_info = '0.91b'
    raxml_info = 'v8.2.12'
    fasttree_info = '2.1.9'
    iqtree_info = '1.6.9'
    ASTRAL_info ='5.6.3'
    SPRSupertrees_info = '1.2.1'
    author_info = 'Author: Yang Fang\n'
    print (VERSION_DESC)
    print (author_info)
    print ("PhySpeTree call follow software, if you use PhySpetree build tree please don't forget cite this:\n ")
    print ("============================CALL SOFTWARE INFO=============================================")
    print ('muscle version: '), (muscle_info + '\ncite:')
    print (citation['muscle'])
    print ('clustalw2 versin: '), (clustalw2_info + '\ncite:')
    print (citation['clustalw2'])
    print ('mafft versin: '), (mafft_info + '\ncite:')
    print (citation['mafft'])
    print ('RAxML version: '), (raxml_info + '\ncite:')
    print (citation['RAxML'])
    print ('FastTree version: '), (fasttree_info + '\ncite:')
    print (citation['FastTree'])
    print ('iqtree version: '), (iqtree_info + '\ncite:')
    print (citation['iqtree'])
    print ('Gblocks version: '), (gblocks_info + '\ncite:')
    print (citation['Gblocks'])
    print ('trimal versin: '), (trimal_info + '\ncite:')
    print (citation['trimal'])
    print ('ASTRAL versin: '), (ASTRAL_info + '\ncite:')
    print (citation['ASTRAL'])
    print ('SPRSupertrees versin: '), (SPRSupertrees_info + '\ncite:')
    print (citation['SPRSupertrees'])
    print ("===========================CALL SOFTWARE INFO==============================================")


citation = {
    'muscle': u"""Edgar R C. MUSCLE: multiple sequence alignment with high accuracy and
    high throughput[J]. Nucleic acids research, 2004, 32(5): 1792-1797.
    """,

    'clustalw2': u"""Larkin M A, Blackshields G, Brown N P, et al. Clustal W and Clustal X
    version 2.0[J]. bioinformatics, 2007, 23(21): 2947-2948.
    """,

    'RAxML': u"""Stamatakis A. RAxML version 8: a tool for phylogenetic analysis and
    post-analysis of large phylogenies[J]. Bioinformatics, 2014, 30(9): 1312-1313.
    """,
    'FastTree': u"""Price M N, Dehal P S, Arkin A P. FastTree 2–approximately
    maximum-likelihood trees for large alignments[J]. PloS one, 2010, 5(3): e9490.""",

    'Gblocks': u"""Talavera G, Castresana J. Improvement of phylogenies after removing
    divergent and ambiguously aligned blocks from protein sequence alignments[J].
    Systematic biology, 2007, 56(4): 564-577.
    """,
    'trimal': u"""Capella-Gutiérrez S, Silla-Martínez J M, Gabaldón T. trimAl: a tool for
     automated alignment trimming in large-scale phylogenetic analyses[J]. Bioinformatics,
      2009, 25(15): 1972-1973.""",
    'mafft': u"""Katoh K, Standley D M. MAFFT multiple sequence alignment software version 7:
     improvements in performance and usability[J]. Molecular biology and evolution, 2013, 30(4):
      772-780.""",
     'iqtree': u"""Nguyen L T, Schmidt H A, von Haeseler A, et al. IQ-TREE: a fast and effective 
     stochastic algorithm for estimating maximum-likelihood phylogenies[J]. Molecular biology and 
     evolution, 2014, 32(1): 268-274.""",
    'ASTRAL': u"""Mirarab S, Reaz R, Bayzid M S, et al. ASTRAL: genome-scale coalescent-based 
    species tree estimation[J]. Bioinformatics, 2014, 30(17): i541-i548.
    """,
    'SPRSupertrees': u"""Whidden C, Zeh N, Beiko R G. Supertrees based on the subtree prune-and-regraft distance[J].
     Systematic biology, 2014, 63(4): 566-581."""
}

if __name__ == '__main__':
    version_infor()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Gblocks module: do gblocks before aligned sequences.

"""
import os
import subprocess
import re
from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath

loggblocks = getLogging('Gblocks')


# Gblocks protein_alignment.fasta -t=p -e=-gb1 -b4=5 -d=y

def dogblocks(indata, gblockpara):
    """
    do gblocks after muslce and concatenate
    :param indata: a fasta file input after gblock
    :param gblockpara: the gblocks para
    :return: a file path of gblocks result
    """
    # Deal with outdata name
    gblockparas = gblockpara.lstrip()
    gblockparalist = gblockparas.split(" ")
    regex = '-e='
    for i in range(0, len(gblockparalist)):
        if re.search(regex, gblockparalist[i]):
            index = i
            break
    outnamepara = gblockparalist[index]
    outdata = outnamepara.split('=')[1]
    gblockpath = getlocalpath()
    alg_name = os.path.basename(indata)
    out_path = os.path.dirname(indata)
    gblock_name = alg_name + outdata
    gblock_data = os.path.join(out_path, gblock_name)

    cmd = gblockpath + "/Gblocks " + indata + " " + gblockparas
    subprocess.call(cmd, shell=True)
    loggblocks.info('Select conserved blocks by Gblocks was completed')
    loggblocks.debug('Gblocks path:{0}'.format(gblock_data))
    return gblock_data



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The main module as enter point and invoke other
script as pipeline.
"""
import time

from physpetool.phylotree.doclustalw import doclustalw_file, doclustalw
from physpetool.phylotree.dofasttree import doFastTree
from physpetool.phylotree.doiqtree import doiqtree
from physpetool.phylotree.domafft import domafft, domafft_file
from physpetool.phylotree.domuscle import domuscle_file, domuscle
from physpetool.phylotree.dogblocks import dogblocks
from physpetool.phylotree.doraxml import doraxml
from physpetool.convert.fasta2phy import fasta2phy
from physpetool.convert.concatenate import cocat_path
from physpetool.phylotree.dotrimal import dotrimal
from physpetool.phylotree.log import getLogging, setlogdir
from physpetool.phylotree.retrievessurna import retrieve16srna
from physpetool.phylotree.retrieveprotein import doretrieve
from physpetool.utils.checkinputfile import checkKeggOrganism, checkSilvaOrganism, checkFile, recovery, recovery_silva
import argparse
import os

APP_DESC = "Reconstruct"

raxmlpara_pro = "-f a -m PROTGAMMAJTTX  -p 12345 -x 12345 -# 100 -n T1"
raxmlpara_dna = "-f a -m GTRGAMMA  -p 12345 -x 12345 -# 100 -n T1"
musclepara = '-maxiters 100'
gblockspara_pro = '-t=p -b5=h -e=-gb1'
gblockspara_dna = '-t=d -b5=h -e=-gb1'
clustalwpara = None
trimalpara = "-gt 1"
mafftpara = "--auto"

auto_build_log = getLogging('Used time')


def start_args(input):
    """
Argument parse
    :param input: arguments
    """
    autobuild_args = input.add_argument_group("AUTOBUILD OPTIONS")
    advance_args = input.add_argument_group("ADVANCE OPTIONS")
    autobuild_args.add_argument('-i', nargs='?', dest='spenames', type=argparse.FileType('r'),
                                help='Input a TXT file contain the species names (abbreviated names) are same with KEGG \
                                species abbreviation.')
    autobuild_args.add_argument('-o', action='store', dest="outdata",
                                default='Outdata',
                                help="A directory include output data (tree files). The default name is \
                                Outdata.")
    autobuild_args.add_argument('-t', action='store', dest='thread',
                                type=int, default=1,
                                help="Specify the number of processing threads (CPUs) to reconstruct \
                                phylogenetic tree. The default is 1.")
    autobuild_args.add_argument('-e', action='store', dest="extenddata",
                                help="The extended data should be FASTA format to extend phylogenetic tree by \
                                     --ehcp or --esrna option.")
    autobuild_args.add_argument('--hcp', action='store_true', dest='HCP',
                                default=True,
                                help="Specify the hcp (highly conserved protein) method to reconstruct \
                                phylogenetic tree. The default method is hcp.")
    autobuild_args.add_argument('--ehcp', action='store_true', dest='EHCP',
                                default=False,
                                help="The ehcp mode is use highly conserved proteins with extend highly  \
                                     conserved protein (users provide) to reconstruct phylogenetic tree.")
    autobuild_args.add_argument('--srna', action='store_true', dest='ssurna',
                                default=False,
                                help="The srna (SSU rRNA) method is use SSU rRNA data to reconstruct \
                               phylogenetic tree.")
    autobuild_args.add_argument('--esrna', action='store_true', dest='essurna',
                                default=False,
                                help="The esrna mode is use SSU RNA sequence with extend SSU RNA sequence \
                                (users provide) to reconstruct phylogenetic tree. ")
    autobuild_args.add_argument('-db', action='store', dest="db",
                                default="",
                                help="The absolute path for local database.")
    advance_args.add_argument('--muscle', action='store_true', dest='muscle',
                              default=True,
                              help="Multiple sequence alignment by muscle. The default multiple sequence \
                                   alignment software is Muscle.")
    advance_args.add_argument('--muscle_p', action='store', dest='muscle_parameter',
                              default=musclepara, help="Set Muscle advance parameters. The default is -maxiter 100.")
    advance_args.add_argument('--clustalw', action='store_true', dest='clustalw',
                              default=False, help="Multiple sequence alignment by clustalw2.")
    advance_args.add_argument('--clustalw_p', action='store', dest='clustalw_parameter',
                              help='Set clustalw2 advance parameters. Here use clustalw default parameters.')
    advance_args.add_argument('--mafft', action='store_true', dest='mafft',
                              default=False, help="Multiple sequence alignment by mafft.")
    advance_args.add_argument('--mafft_p', action='store', dest='mafft_parameter',
                              default=mafftpara,
                              help='Set mafft advance parameters. Here use mafft default parameters.')
    advance_args.add_argument('--gblocks', action='store_true', dest='gblocks',
                              default=True, help="Trim by Gblocks.")
    advance_args.add_argument('--gblocks_p', action='store', dest='gblocks_parameter',
                              default=gblockspara_pro, help="Set Gblocks advance parameters.")
    advance_args.add_argument('--trimal', action='store_true', dest='trimal',
                              default=False, help="Trim by trimal.")
    advance_args.add_argument('--trimal_p', action='store', dest='trimal_parameter',
                              default=trimalpara, help="Set trimal advance parameters.")
    advance_args.add_argument('--raxml', action='store_true', dest='raxml', default=True,
                              help="Reconstruct phylogenetic tree by RAxML. The default build tree software is RAxML.")
    advance_args.add_argument('--raxml_p', action='store', dest='raxml_parameter',
                              default=raxmlpara_pro, help='Set RAxML advance parameters. ')
    advance_args.add_argument('--fasttree', action='store_true', dest='fasttree',
                              default=False, help='Reconstruct phylogenetic tree by FastTree.')
    advance_args.add_argument('--fasttree_p', action='store', dest='fasttree_parameter',
                              default='', help='Set FastTree advance parameters. ')
    advance_args.add_argument('--iqtree', action='store_true', dest='iqtree',
                              default=False, help='Reconstruct phylogenetic tree by iqtree.')
    advance_args.add_argument('--iqtree_p', action='store', dest='iqtree_parameter',
                              default='', help='Set iqtree advance parameters. ')


def starting(args):
    """
starting run reconstruct tree
    :param args: arguments
    """
    print("Loading organism's names success.....\n")
    print("The result are store in:{0}\n".format(args.outdata))
    print("Now loading data and constructing phylogenetic tree......")

    in_put = args.spenames

    pwd = os.getcwd()

    out_put = os.path.join(pwd, args.outdata)
    if args.extenddata:
        if os.path.isfile(args.extenddata):
            args_extend = args.extenddata
        elif os.path.isdir(args.extenddata):
            args_extend = os.path.join(pwd, args.extenddata)
    else:
        pass

    # Reconstruct phylogenetic tree by ssu RNA.
    if args.ssurna:
        setlogdir(out_put)
        starting_srna(in_put, out_put,
                      args.muscle, args.muscle_parameter,
                      args.clustalw, args.clustalw_parameter,
                      args.mafft, args.mafft_parameter,
                      args.gblocks, args.gblocks_parameter,
                      args.trimal, args.trimal_parameter,
                      args.raxml, args.raxml_parameter,
                      args.fasttree, args.fasttree_parameter,
                      args.iqtree, args.iqtree_parameter,
                      args.thread,args.db)

    # Reconstruct phylogenetic tree by extend highly conserved proteins.
    elif args.EHCP:
        setlogdir(out_put)
        starting_ehcp(in_put, out_put,
                      args.muscle, args.muscle_parameter,
                      args.clustalw, args.clustalw_parameter,
                      args.mafft, args.mafft_parameter,
                      args.gblocks, args.gblocks_parameter,
                      args.trimal, args.trimal_parameter,
                      args.raxml, args.raxml_parameter,
                      args.fasttree, args.fasttree_parameter,
                      args.iqtree, args.iqtree_parameter,
                      args.thread, args.extenddata,
                      args.db)

    # Reconstruct phylogenetic tree by extend ssu rna method.
    elif args.essurna:
        setlogdir(out_put)
        starting_esrna(in_put, out_put,
                       args.muscle, args.muscle_parameter,
                       args.clustalw, args.clustalw_parameter,
                       args.mafft, args.mafft_parameter,
                       args.gblocks, args.gblocks_parameter,
                       args.trimal, args.trimal_parameter,
                       args.raxml, args.raxml_parameter,
                       args.fasttree, args.fasttree_parameter,
                       args.iqtree, args.iqtree_parameter,
                       args.thread, args.extenddata,
                       args.db)
    # Reconstruct phylogenetic tree by highly conserved proteins.
    elif args.HCP:
        setlogdir(out_put)
        starting_hcp(in_put, out_put,
                     args.muscle, args.muscle_parameter,
                     args.clustalw, args.clustalw_parameter,
                     args.mafft, args.mafft_parameter,
                     args.gblocks, args.gblocks_parameter,
                     args.trimal, args.trimal_parameter,
                     args.raxml, args.raxml_parameter,
                     args.fasttree, args.fasttree_parameter,
                     args.iqtree, args.iqtree_parameter,
                     args.thread, args.db)


def starting_hcp(in_put, out_put,
                 args_muscle, args_muscle_p,
                 args_clustalw, args_clustalw_p,
                 args_mafft, args_mafft_p,
                 args_gblocks, args_gblocks_p,
                 args_trimal, args_trimal_p,
                 args_raxml, args_raxml_p,
                 args_fasttree, args_fasttree_p,
                 args_iqtree, args_iqtree_p,
                 args_thread, args_db):
    '''reconstruct phylogenetic tree by hcp method'''
    hcp_input, recovery_dic = checkKeggOrganism(in_put)
    start = time.time()
    out_retrieve = doretrieve(hcp_input, out_put,args_db)
    end = time.time()
    auto_build_log.info('Retrieving HCP sequences used time: {} Seconds'.format(end - start))

    if not recovery_dic == {}:
        recovery(out_retrieve, recovery_dic)
    # set default aligned by muscle if not specify clustalw
    start2 = time.time()
    if args_clustalw:
        out_alg = doclustalw_file(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft_file(out_retrieve, out_put, args_mafft_p, args_thread)
    elif args_muscle:
        out_alg = domuscle_file(out_retrieve, out_put, args_muscle_p, args_thread)

    out_concat = cocat_path(out_alg)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_concat, args_trimal_p)
    elif args_gblocks:
        out_gblock = dogblocks(out_concat, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)

    # reconstruct tree
    if args_fasttree:
        doFastTree(out_f2p, out_put, args_fasttree_p, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end2 = time.time()
    auto_build_log.info('Contracting species tree used time: {} Seconds'.format(end2 - start2))


def starting_srna(in_put, out_put,
                  args_muscle, args_muscle_p,
                  args_clustalw, args_clustalw_p,
                  args_mafft, args_mafft_p,
                  args_gblocks, args_gblocks_p,
                  args_trimal, args_trimal_p,
                  args_raxml, args_raxml_p,
                  args_fasttree, args_fasttree_p,
                  args_iqtree, args_iqtree_p,
                  args_thread,args_db):
    '''reconstruct phylogenetic tree by ssu rna method'''
    ssu_input, recovery_dic = checkSilvaOrganism(in_put)
    start = time.time()
    out_retrieve = retrieve16srna(ssu_input, out_put,args_db)
    end = time.time()
    auto_build_log.info('Retrieving SSU rRNA sequences used time: {} Seconds'.format(end - start))
    if not recovery_dic == []:
        recovery_silva(out_retrieve, recovery_dic, ssu_input)

    # set default aligned by muscle if not specify clustalw or mafft
    start2 = time.time()
    if args_clustalw:
        out_alg = doclustalw(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft(out_retrieve, out_put, args_mafft_p)
    elif args_muscle:
        out_alg = domuscle(out_retrieve, out_put, args_muscle_p)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_alg, args_trimal_p)
    elif args_gblocks:
        if args_gblocks_p is gblockspara_pro:
            args_gblocks_p = gblockspara_dna
            out_gblock = dogblocks(out_alg, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)

    # reconstruct tree
    if args_fasttree:
        args_fasttree_p_add = "-nt " + args_fasttree_p.lstrip()
        doFastTree(out_f2p, out_put, args_fasttree_p_add, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        if args_raxml_p is raxmlpara_pro:
            args_raxml_p = raxmlpara_dna
            doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end2 = time.time()
    auto_build_log.info('Constructing species tree used time: {} Seconds'.format(end2 - start2))


def starting_ehcp(in_put, out_put,
                  args_muscle, args_muscle_p,
                  args_clustalw, args_clustalw_p,
                  args_mafft, args_mafft_p,
                  args_gblocks, args_gblocks_p,
                  args_trimal, args_trimal_p,
                  args_raxml, args_raxml_p,
                  args_fasttree, args_fasttree_p,
                  args_iqtree, args_iqtree_p,
                  args_thread, args_extenddata,
                  args_db):
    '''reconstruct phylogenetic tree by ehcp method'''
    hcp_input, recovery_dic = checkKeggOrganism(in_put)
    start = time.time()
    out_retrieve = doretrieve(hcp_input, out_put,args_db)
    end = time.time()
    auto_build_log.info('Retrieving HCP sequences used time: {} Seconds'.format(end - start))

    if not recovery_dic == {}:
        recovery(out_retrieve, recovery_dic)
    retrieve_pro = os.listdir(out_retrieve)
    for reline in retrieve_pro:
        fw_name = os.path.join(out_retrieve, reline)
        fr_name = os.path.join(args_extenddata, reline)
        fw = open(fw_name, 'a')
        with open(fr_name) as fr:
            for line in fr:
                fw.write(line)
        fw.close()

    # set default aligned by muscle if not specify clustalw or mafft
    start2 = time.time()

    if args_clustalw:
        out_alg = doclustalw_file(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft_file(out_retrieve, out_put, args_mafft_p, args_thread)
    elif args_muscle:
        out_alg = domuscle_file(out_retrieve, out_put, args_muscle_p, args_thread)

    out_concat = cocat_path(out_alg)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_concat, args_trimal_p)
    elif args_gblocks:
        out_gblock = dogblocks(out_concat, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)
    # reconstruct tree
    if args_fasttree:
        doFastTree(out_f2p, out_put, args_fasttree_p, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end2 = time.time()
    auto_build_log.info('Constructing species tree used time: {} Seconds'.format(end2 - start2))

def starting_esrna(in_put, out_put,
                   args_muscle, args_muscle_p,
                   args_clustalw, args_clustalw_p,
                   args_mafft, args_mafft_p,
                   args_gblocks, args_gblocks_p,
                   args_trimal, args_trimal_p,
                   args_raxml, args_raxml_p,
                   args_fasttree, args_fasttree_p,
                   args_iqtree, args_iqtree_p,
                   args_thread, args_extenddata,
                   args_db):
    '''reconstruct phylogenetic tree by ssu rna extend method'''
    extend_check = checkFile(args_extenddata)
    ssu_input, recovery_dic = checkSilvaOrganism(in_put)
    start = time.time()
    out_retrieve = retrieve16srna(ssu_input, out_put,args_db)
    end = time.time()
    auto_build_log.info('Retrieving SSU rRNA sequences used time: {} Seconds'.format(end - start))
    if not recovery_dic == []:
        recovery_silva(out_retrieve, recovery_dic, ssu_input)
    retrieve_srna_path = os.path.join(out_retrieve, 'rna_sequence.fasta')

    fw = open(retrieve_srna_path, 'a')
    with open(extend_check) as read:
        for line in read:
            fw.write(line)
    fw.close()

    # set default aligned by muscle if not specify clustalw
    start2 = time.time()
    if args_clustalw:
        out_alg = doclustalw(out_retrieve, out_put, args_clustalw_p)
    elif args_mafft:
        out_alg = domafft(out_retrieve, out_put, args_mafft_p)
    elif args_muscle:
        out_alg = domuscle(out_retrieve, out_put, args_muscle_p)

    # set default trim by gblocks if not specify trimal
    if args_trimal:
        out_f2p = dotrimal(out_alg, args_trimal_p)
    elif args_gblocks:
        if args_gblocks_p is gblockspara_pro:
            args_gblocks_p = gblockspara_dna
            out_gblock = dogblocks(out_alg, args_gblocks_p)
        out_f2p = fasta2phy(out_gblock)
    # reconstruct tree
    if args_fasttree:
        args_fasttree_p_add = "-nt " + args_fasttree_p.lstrip()
        doFastTree(out_f2p, out_put, args_fasttree_p_add, args_thread)
    elif args_iqtree:
        doiqtree(out_f2p, out_put, args_iqtree_p, args_thread)
    elif args_raxml:
        if args_raxml_p is raxmlpara_pro:
            args_raxml_p = raxmlpara_dna
            doraxml(out_f2p, out_put, args_raxml_p, args_thread)
    end2 = time.time()
    auto_build_log.info('Constructing species tree used time: {} Seconds'.format(end2 - start2))



# ########################## Copyrights and License ############################
#                                                                              #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                             #
#                                                                              #
# This file is part of Physpe.                                                 #
# https://xiaofeiyangyang.github.io/physpetools/                               #
#                                                                              #
# Physpe is free software: you can redistribute it and/or modify it under      #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# Physpe is distributed in the hope that it will be useful, but WITHOUT ANY    #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with Physpe. If not, see <http://www.gnu.org/licenses/>.               #
#                                                                              #
# ##############################################################################

"""
Create the kegg ko index database

"""
from urllib import request

from physpetool.tools.operateDB import operate_db

ko_db = operate_db('/home/yangfang/PhySpeTree/physpetools/physpetool/database/KEGG_DB_3.0.db')


def create_table_pro():
    """" create the table of highly conserved protein index"""
    table_name = "proindex"
    listko = []
    with open('/home/yangfang/PhySpeTree/physpetools/physpetool/database/protein_ko.txt') as org:
        for line in org:
            name = line.strip().split(',')
            listko.append(name[1])
    ko_db.create(table_name, listko)
    ko_db.dispay_col('proindex', 'NAME')


def insert_name():
    """insert the abb names"""
    orgs = []
    with open('/home/yangfang/PhySpeTree/physpetools/physpetool/database/organism.txt') as org:
        for line in org:
            strs = line.strip().split('\t')
            orgs.append(strs[1])
    ko_db.insert('proindex', orgs)


def get_abb():
    """Get the organism abb names"""
    euk = []
    prokar = []
    with open('/home/yangfang/PhySpeTree/physpetools/physpetool/database/organism.txt') as org:
        for line in org:
            strs = line.strip().split('\t')

            if strs[3].split(';')[0] == 'Eukaryotes':
                euk.append(strs[1])
            else:
                prokar.append(strs[1])
    return euk, prokar


def get_ko_list():
    """Gte eukaryotes and porkarytes KO list"""
    euk_ko = []
    prokar_ko = []
    with open('/home/yangfang/PhySpeTree/physpetools/physpetool/database/protein_ko.txt') as ko:
        for line in ko:
            strs = line.strip().split(',')
            euk_ko.append(strs[1])
            prokar_ko.append(strs[2])
    return euk_ko, prokar_ko


def getkolist(ko):
    """Get KO list"""
    listko = []
    url = "http://rest.kegg.jp/link/genes/" + ko

    page = request.urlopen(url).read()
    page = page.decode('utf-8')
    page = page.split('\n')
    page.remove('')
    for line in page:
        html = line.strip().split('\t')
        listko.append(html)
    return listko


def update_euk():
    '''Update eukaryote kegg protein index'''
    euk, prokar = get_abb()
    euk_ko, prokar_ko = get_ko_list()
    for line in euk_ko:
        kolist = getkolist(line)
        for ko in kolist:
            value = ko[1]
            list_abb = value.split(':')[0]
            if list_abb in euk:
                print('proindex', line, list_abb, value)
                ko_db.update('proindex', line, list_abb, value)


def update_prokar():
    """Update prokaryotes kegg protein index"""
    euk, prokar = get_abb()
    euk_ko, prokar_ko = get_ko_list()
    len_euk_ko = len(euk_ko)
    for line in range(len_euk_ko):
        kolist = getkolist(prokar_ko[line])
        for ko in kolist:
            value = ko[1]
            list_abb = value.split(':')[0]
            if list_abb in prokar:
                print('proindex', euk_ko[line], list_abb, value)
                ko_db.update('proindex', euk_ko[line], list_abb, value)


def display():
    ko_db.display_row('proindex', 'hsa')


if __name__ == '__main__':
    # Create table
    create_table_pro()
    # insert abb name
    insert_name()
    # Update eukaryotes protein index
    update_euk()
    # Update prokaryotes protein index
    update_prokar()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The arguments parse module for build module.

"""

import os
from physpetool.phylotree.buildtree import build_hcp, build_srna
from physpetool.phylotree.log import setlogdir

APP_DESC = ""

raxmlpara_pro = "-f a -m PROTGAMMAJTTX  -p 12345 -x 12345 -# 100 -n T1"
raxmlpara_dna = "-f a -m GTRGAMMA  -p 12345 -x 12345 -# 100 -n T1"
musclepara = "-maxiters 100"
gblockspara_pro = '-t=p -b5=h -e=-gb1'
gblockspara_dna = '-t=d -b5=h -e=-gb1'
clustalwpara = None
trimalpara = "-gt 1"
mafftpara = "--auto"


def start_args(input):
    """
Arguments parse
    :param input:  Arguments
    """
    build_args = input.add_argument_group("BUILD OPTIONS")
    advance_args = input.add_argument_group("ADVANCE OPTIONS")
    build_args.add_argument('-i', action='store', dest='input',
                            help="Input file FASTA format for '--sran' method or a directory contain conserved\
                                proteins for '--hcp' method.")
    build_args.add_argument('-o', action='store', dest="outdata",
                            default='Outdata',
                            help="A directory include output data (tree files). The default name is \
                                Outdata.")
    build_args.add_argument('-t', action='store', dest='thread',
                            type=int, default=1,
                            help="Specify the number of processing threads (CPUs) to reconstruct \
                                phylogenetic tree. The default is 1.")
    build_args.add_argument('--multiple', action='store_true', dest='HCP',
                            default=True,
                            help="Specify concatenate highly conserved protein method to reconstruct \
                                phylogenetic tree. The default method.")
    build_args.add_argument('--single', action='store_true', dest='ssurna',
                            default=False,
                            help="Use SSU rRNA data to reconstruct \
                               phylogenetic tree.")
    advance_args.add_argument('--muscle', action='store_true', dest='muscle',
                              default=True,
                              help="Multiple sequence alignment by muscle. The default multiple sequence \
                                   alignment software is Muscle.")
    advance_args.add_argument('--muscle_p', action='store', dest='muscle_parameter',
                              default=musclepara, help="Set Muscle advance parameters. The default is -maxiter 100.")
    advance_args.add_argument('--clustalw', action='store_true', dest='clustalw',
                              default=False, help="Multiple sequence alignment by clustalw2.")
    advance_args.add_argument('--clustalw_p', action='store', dest='clustalw_parameter',
                              help='Set clustalw2 advance parameters. Here use clustalw default parameters.')
    advance_args.add_argument('--mafft', action='store_true', dest='mafft',
                              default=False, help="Multiple sequence alignment by mafft.")
    advance_args.add_argument('--mafft_p', action='store', dest='mafft_parameter',
                              default=mafftpara,
                              help='Set mafft advance parameters. Here use mafft default parameters.')
    advance_args.add_argument('--gblocks', action='store_true', dest='gblocks',
                              default=True, help="Trim by Gblocks.")
    advance_args.add_argument('--gblocks_p', action='store', dest='gblocks_parameter',
                              default=gblockspara_pro, help="Set Gblocks advance parameters.")
    advance_args.add_argument('--trimal', action='store_true', dest='trimal',
                              default=False, help="Trim by trimal.")
    advance_args.add_argument('--trimal_p', action='store', dest='trimal_parameter',
                              default=trimalpara, help="Set trimal advance parameters.")
    advance_args.add_argument('--raxml', action='store_true', dest='raxml', default=True,
                              help="Reconstruct phylogenetic tree by RAxML. The default build tree software is RAxML.")
    advance_args.add_argument('--raxml_p', action='store', dest='raxml_parameter',
                              default=raxmlpara_pro, help='Set RAxML advance parameters. ')
    advance_args.add_argument('--fasttree', action='store_true', dest='fasttree',
                              default=False, help='Reconstruct phylogenetic tree by FastTree.')
    advance_args.add_argument('--fasttree_p', action='store', dest='fasttree_parameter',
                              default='', help='Set FastTree advance parameters. ')
    advance_args.add_argument('--iqtree', action='store_true', dest='iqtree',
                              default=False, help='Reconstruct phylogenetic tree by iqtree.')
    advance_args.add_argument('--iqtree_p', action='store', dest='iqtree_parameter',
                              default='', help='Set iqtree advance parameters. ')


def starting(args):
    """
Staring run build
    :param args: arguments
    """

    pwd = os.getcwd()

    out_put = os.path.join(pwd, args.outdata)
    if args.input:
        if os.path.isfile(args.input):
            args_input = args.input
        elif os.path.isdir(args.input):
            args_input = os.path.join(pwd, args.input)


    # reconstruct phylogenetic tree by ssu RNA
    if args.ssurna:
        setlogdir(out_put)
        build_srna(args_input, out_put,
                   args.muscle, args.muscle_parameter,
                   args.clustalw, args.clustalw_parameter,
                   args.mafft, args.mafft_parameter,
                   args.gblocks, args.gblocks_parameter,
                   args.trimal, args.trimal_parameter,
                   args.raxml, args.raxml_parameter,
                   args.fasttree, args.fasttree_parameter,
                   args.iqtree, args.iqtree_parameter,
                   args.thread)
    elif args.HCP:
        setlogdir(out_put)
        build_hcp(args_input, out_put,
                  args.muscle, args.muscle_parameter,
                  args.clustalw, args.clustalw_parameter,
                  args.mafft, args.mafft_parameter,
                  args.gblocks, args.gblocks_parameter,
                  args.trimal, args.trimal_parameter,
                  args.raxml, args.raxml_parameter,
                  args.fasttree, args.fasttree_parameter,
                  args.iqtree, args.iqtree_parameter,
                  args.thread)


import sqlite3
import os
from Bio import SeqIO

db ="/home/yangfang/PhySpeTree/physpetools/physpetool/database/KEGG_DB_3.0.db"
conn = sqlite3.connect(db)
conn.text_factory = str
c = conn.cursor()
c.execute("select * from proindex")
x = c.fetchall()


def read_seq(seq_path):
    seq_db = {}

    reads = SeqIO.parse(seq_path, 'fasta')
    for read in reads:
        seq = str(read.seq)
        seq_name = str(read.name)
        tem = seq_name.strip().split("|")
        if len(tem) > 1:
            seq_name = tem[1]
        seq_db[seq_name] = seq
    return seq_db


total = []
not_have = []

HCP_db = "/home/yangfang/PhySpeTree/revision/HCPDB/hcpdb/"
db_path = "/home/yangfang/PhySpeTree/revision/HCPDB/abb_seqs1000/"
all_ = len(x)
idx =1
for line in x:
    tem = [var for var in line if var != None]
    spe_name = tem[1]
    seq_name = tem[2:]
    read_path = db_path + spe_name + ".fasta"
    if os.path.isfile(read_path):
        seqs = read_seq(read_path)
        for each in seq_name:
            if each in seqs.keys():
                w_path = HCP_db + each + ".fasta"
                # print(w_path)
                f_w = open(w_path,'w')
                f_w.write(">" + each +"\n" + seqs[each])
                f_w.close()
            else:
                not_have.append(each)
    else:
        not_have.extend(seq_name)
    idx +=1
    print(idx/all_)


with open("/home/yangfang/PhySpeTree/revision/HCPDB/hcp_code.txt",'w') as f:
    for each in not_have:
        f.write(each +"\n")





import os
from urllib import request
import time
def splist(l, s):
    """split a list to sub list contain s"""
    return [l[i:i + s] for i in range(len(l)) if i % s == 0]

def getprotein(proid):
    # http://rest.kegg.jp/get/hsa:10458+ece:Z5100+pon:100172290
    url = "http://rest.kegg.jp/get/" + proid + "/aaseq"

    page = request.urlopen(url,timeout=60)
    time.sleep(5)
    return page


d_seq = []

with open("/home/yangfang/PhySpeTree/revision/HCPDB/hcp_code.txt") as f:
    for line in f:
        d_seq.append(line.strip())

all_file_path = '/home/yangfang/PhySpeTree/revision/HCPDB/hcp_append/'
all_files = list(filter(lambda f: not f.startswith('.'), os.listdir(all_file_path)))

for line in all_files:
    name = line.split('.')[0]
    if ( name in d_seq):
        d_seq.remove(name)

spprolist = splist(d_seq, 10)

idx =1
all_ = len(spprolist)

for id in spprolist:
    print(idx / all_)
    res = {}
    get_abb = []
    id_abb = [">"+x.strip().split(" ")[0]for x in id]
    query = "+".join(id)
    protein = getprotein(query)
    protein = protein.readlines()
    protein = [x.decode('utf-8') for x in protein]
    for i in protein:
        if i.startswith(">"):

            abbname = i.strip().split(" ")[0]
            get_abb.append(abbname)

            print(abbname)
            res[abbname] = []
            # print(abbname)
        else:
            res[abbname].append(i)
    # if the abb protein code changed by KEGG DB
    if len(get_abb) != len(id_abb):
        loss_abb = [x for x in id_abb if x not in get_abb]
        for loss in loss_abb:
            res[loss] = "M\n"


    for k, v in res.items():
        w_file = all_file_path + k + ".fasta"
        w_path = open(w_file, "w")
        w_path.write(k+"\n" + "".join(v))
        w_path.close()
    idx +=1


# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################


"""
The module for check highly conserved proteins will be prepared.
"""

from physpetool.phylotree.retrieveprotein import getcolname, getspecies, hcp_name
from physpetool.utils.checkinputfile import checkFile, readIputFile, checkKeggOrganism, checkSilvaOrganism, \
    check_organism
import os

from physpetool.utils.printstyle import print_style


def check_ehcp(input, output):
    """
    Check the highly conserved proteins will be prepared.
    :param input: the species abbreviated names
    :param output: a directory contain check result
    """
    # prepare a file write result
    if not os.path.exists(output):
        os.makedirs(output)
    fw_name = "PhySpeTree_echp_extend.txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')

    # check input names
    input_path = checkFile(input)
    input_list = readIputFile(input_path)
    colname = getcolname()
    relist, match_ko = getspecies(input_list, colname)
    p = 1
    for line in match_ko:
        hcpname = hcp_name(line.strip())
        massage = "'{0}' ----------------------------------> p{1}.fasta\n".format(hcpname, str(p))
        print(massage)
        fw.write(massage)
        p += 1
    print("Checked extend highly conserved proteins is completed.")
    print("Checked result was store in {0}".format(open_path))
    fw.close()


def check_hcp(input, output):
    """
    Check whether the input species in the hcp database
    :param input: the species names
    :param output: the directory contains check result
    """
    if not os.path.exists(output):
        os.makedirs(output)
    fw_name = "PhySpeTree_hcp_checked.txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')

    input_path = checkFile(input)
    input_list = readIputFile(input_path)

    in_match, no_match = check_organism(input_list, "organism_kegg_to_tax.txt")

    if no_match.__len__() == 0:
        print(print_style('INFO: ', fore='green') + "All species are match in KEGG DATABASE.")
        fw.write("All species are match in KEGG DATABASE")
    else:
        print(print_style('WARNING: ', fore='red') + "The following species are not supported by KEGG DATABASE:")
        fw.write("The following species are not supported by KEGG DATABASE:\n")
        for line in no_match:
            print(line)
            fw.write(line + "\n")

    print("Checked  whether the input species names in KEGG DATABASE completed.")
    print("Checked result is store in {0}".format(open_path))
    fw.close()


def check_srna(input, output):
    """
    Check whether the input species in the SSU rRNA database
    :param input: the species names
    :param output: the directory contains check result
    """
    if not os.path.exists(output):
        os.makedirs(output)
    fw_name = "PhySpeTree_srna_checked.txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')

    input_path = checkFile(input)
    input_list = readIputFile(input_path)

    in_match, no_match = check_organism(input_list, "kegg_to_silva_id.txt")

    if no_match.__len__() == 0:
        print(print_style('INFO: ', fore='green') + "All species are match in SILVA DATABASE")
        fw.write("All species are match in SILVA DATABASE")
    else:
        print(print_style('WARNING: ', fore='red') + "The following species are not supported by SILVA DATABASE:")
        fw.write("The following species are not supported by SILVA DATABASE:\n")
        for line in no_match:
            print(line)
            fw.write(line + "\n")

    print("Checked  whether the input species names in SILVA DATABASE completed.")
    print("Checked result is store in {0}".format(open_path))
    fw.close()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Create highly conserved proteins index databse.

"""

import sqlite3
import time

from physpetool.tools.keggapi import getkolist


def getdata(database, name):
    """
    get the data by db contain eukaryotes prokaryotes KO number
    :param database: the databath path
    :param name: database col name
    :return: the query data from database
    """
    conn = sqlite3.connect(database)
    sqlselect = "select {0} from KOINDEX".format(name)
    conn.text_factory = str
    c = conn.cursor()
    c.execute(sqlselect)
    konum = c.fetchall()
    c.close()
    return konum


def insertdata(konum, abbname, value):
    """
    Insert protein index to  database
    example: insertdata('K02865','hsa','has:2525525')
    :param konum: the protein KO number
    :param abbname: the species abbreviation by kegg
    :param value: protein id by keegg
    """
    conn = sqlite3.connect('../database/KEGG_DB_1.0.db')
    conn.text_factory = str
    c = conn.cursor()
    sqlupdate = "UPDATE proindex SET {0}=? WHERE NAME=?".format(konum)
    c.execute(sqlupdate, (value, abbname))
    conn.commit()
    c.close()


# c.execute("UPDATE keggproindex SET KO2865='testdata' WHERE NAME='hsa'")
# c.execute("select NAME from keggproindex")
# col_name_list = [tuple[0] for tuple in cursor.description]
# print col_name_list
# for row in c.execute('SELECT * FROM keggproindex'):
#     print row

def kolinkinsertprokar(konum):
    """get Kegg KO link about prokartotes"""
    kolist = getkolist(konum[1])
    for ko in kolist:
        value = ko[1]
        abb = value.split(':')[0]
        insertdata(konum[0], abb, value)


def kolinkinserteuk(konum):
    """get Kegg KO link about eukaryotes"""
    kolist = getkolist(konum)
    for ko in kolist:
        value = ko[1]
        abb = value.split(':')[0]
        insertdata(konum, abb, value)


def inserteuk():
    """insert eukaryotes data from kegg"""
    kodata = getdata('../database/koindex.db', 'EUKARYOTES,PROKARYOTES')
    for line in kodata:
        ko = line[0]
        print (ko)
        kolinkinserteuk(ko)


def insertprokar():
    """insert prokaryotes data form kegg"""
    kodata = getdata('../database/koindex.db', 'EUKARYOTES,PROKARYOTES')
    for line in kodata:
        print (line[1])
        kolinkinsertprokar(line)


def countcol():
    """check the database the protein number index are already get
        and the none data
    """
    countlen = []
    countnone = []
    conn = sqlite3.connect('../database/proindex.db')
    conn.text_factory = str
    c = conn.cursor()
    for row in c.execute('SELECT * FROM keggproindex'):
        listrow = list(row)
        relistrow = [x for x in listrow if x is not None]
        lens = len(relistrow) - 2
        if lens is 0:
            countnone.append(row)
        countlen.append(lens)
    c.close()
    return countlen, countnone


def displaydb():
    """display de database the length, max, min, average et al. of species respectively"""
    countlen, countnone = countcol()
    print ("Total organism in database:", len(countlen))
    print ("Max of get protein number is:", max(countlen))
    print ("Min of get protein number is:", min(countlen))
    print ("Average of get protein number is:", sum(countlen) / len(countlen))
    print ("Number of can't get any protein:", len(countnone))
    print ("Nhe species are can't get any protein names is:")
    for abb in countnone:
        print (abb[1] + ",")


def display_row(abb):
    conn = sqlite3.connect('../database/proindex.db')
    conn.text_factory = str
    c = conn.cursor()
    sqlupdate = "select * from keggproindex  where NAME='{0}'".format(abb)
    c.execute(sqlupdate)
    result = c.fetchall()
    print (result)
    conn.commit()
    c.close()


if __name__ == '__main__':
    # start = time.time()
    # displaydb()
    #
    # end = time.time()
    # during = end - start
    # print "\n"
    # print "the execute time is:"
    # print(during)

    insertdata(konum='K02877', abbname='cit', value='cit:102623164')
    display_row('zma')



# update id =151
# select * from keggproindex where id =151
# update keggproindex set K02925 = 'zma:101027258' where id =151
# commit



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################


"""
Convert FASTA(.fasta) format to PHYLIP(.phy) format

"""
from physpetool.phylotree.log import getLogging

logfasta2phy = getLogging('convert')


# function to read fasta file
def read_fasta(fp):
    """
    read *.fasta file and parse
    :param fp: a fasta format file
    """
    seq_name, seq_pro = None, []
    for line in fp:
        line = line.rstrip()
        if line.startswith(">"):
            if seq_name: yield (seq_name, ''.join(seq_pro))
            seq_name, seq_pro = line.replace(">", ""), []
        else:
            seq_pro.append(line.replace(" ", ""))
    if seq_name: yield (seq_name, ''.join(seq_pro))


def fasta2phy(inputfile):
    """
    convert fasta format file to PHYLIP format
    :param inputfile: *.fasta file
    :return: output data path
    """
    fwfile = inputfile + '.phy'
    fw = open(fwfile, 'w')
    fr = open(inputfile)
    count = 0
    with fr as fp:
        for str in fp.readlines():
            if str.startswith(">"):
                count += 1
        fp.seek(0, 0)
        for seq_name, seq_pro in read_fasta(fp):
            seq_len = len(seq_pro)
        fw.write("%d %d\n" % (count, seq_len))
        fp.seek(0, 0)
        for seq_name, seq_pro in read_fasta(fp):
            fw.write("%s %s\n" % (seq_name, seq_pro))
    logfasta2phy.info("Fasta format converted to PHYLIP format was completed")
    return fwfile



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The arguments parse module for combine module.

"""

import os
from physpetool.phylotree.consensustree import docontree, do_astral, do_supertree

APP_DESC = ""


def start_args(input):
    """
Arguments parse
    :param input:  Arguments
    """
    combine_args = input.add_argument_group("COMBINE OPTIONS")
    combine_args.add_argument('-i', action='store', dest="inputfile",
                              help='iInput a tree file (PHYLIP format), which contain multiple tree.')
    combine_args.add_argument('-o', action='store', dest="outputfile",
                              default='combinetree', help='A directory contain combined tree file. \
                              The default is combineTree.')
    combine_args.add_argument('--mr', action='store_true', dest="mr",
                              default=True, help='Compute majority rule consensus tree. default.')
    combine_args.add_argument('--mre', action='store_true', dest="mre",
                              default=False, help='Compute extended majority rule consensus tree.')
    combine_args.add_argument('--strict', action='store_true', dest="strict",
                              default=False, help='Compute strict consensus tree.')
    combine_args.add_argument('--astral', action='store_true', dest="astral",
                              default=False, help='Use ASTRAL combine multi gene tree.')
    combine_args.add_argument('--supertree', action='store_true', dest="supertree",
                              default=False, help='Use Spr_Supertree combining conflicting evolutionary histories \
                               that are due to lateral gene transfer (LGT).')

def starting(args):
    """
Staring run combine
    :param args: arguments
    """
    pwd = os.getcwd()

    out_put = os.path.join(pwd, args.outputfile)

    # get the consensus tree method
    if args.mre:
        rule = "MRE"
    elif args.strict:
        rule = "STRICT"
    elif args.supertree:
        rule = "supertree"
    elif args.mr:
        rule = "MR"

    # merge tree
    if args.astral:
        do_astral(args.inputfile, out_put)
    elif args.supertree:
        do_supertree(args.inputfile, out_put)
    else:
        docontree(args.inputfile, out_put, rule)



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Deal with SILVA database data for create 16s SSU rRNA database.

"""

import os


def convert_ncbi_id(read, write):
    """
convert taxmap_emble file to accession silva id
    :param read: taxmap emble file
    :param write: write file user rename
    """
    fw = open(write, 'w')
    with open(read) as read:
        for line in read:
            tem = line.strip().split('\t')
            col1 = '{0}.{1}.{2}'.format(tem[0], tem[1], tem[2])
            col2 = tem[-1]
            fw.write(col1 + '\t' + col2 + '\n')
    fw.close()


def convert_ncbi_silva(ncbilist, silvalist):
    """
convert ncbi id to silva accession
    :param ncbilist: a txt file contain ncbi id list
    :param silvalist: a txt file contain silva id list
    """
    fw = open('ncbi_to_silva_id.txt', 'a')
    with open(ncbilist) as read:
        for line in read:
            nid = line.strip().split('\t')
            nid1 = nid[1].strip()
            for silvaid in open(silvalist):
                sid = silvaid.strip().split('\t')
                sid1 =sid[1].strip()
                if nid1 == sid1:
                    print (nid1)
                    fw.write(nid[0] + '\t' + nid[1] + '\t' + sid[0] + '\n')
                    break
                else:
                    pass
    fw.close()


os.chdir('/home/yangfang/physpetools_data/16sRNA')
# convert_ncbi_id('taxmap_embl_ssu_ref_123.1.txt', 'silva_map_ncbi_ref.txt')
convert_ncbi_silva('organism_convert_to_tax.txt', 'silva_map_ncbi_NR.txt')



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
API for KEGG database.
"""
import sys

try:
    from urllib import request
    python_version = sys.version_info[0]
except ImportError:
    import urllib2
    python_version = sys.version_info[0]

import re


# ('Ribosomal protein L1 ', 'K02865')
def getorganismlist():
    if (python_version == 3):
        page = request.urlopen('http://rest.kegg.jp/list/organism')
    else:
        page = urllib2.urlopen('http://rest.kegg.jp/list/organism')
    html = page.read()
    return html


def getkolist(ko):
    """Get KO list"""
    listko = []
    url = "http://rest.kegg.jp/link/genes/" + ko
    if (python_version == 3):
        page = request.urlopen(url).read()
        page = page.decode('utf-8')
        page = page.split('\n')
        page.remove('')
        for line in page:
            html = line.strip().split('\t')
            listko.append(html)
        else:
            page = urllib2.urlopen(url)
            for line in page:
                html = line.strip().split('\t')
                listko.append(html)
    return listko


# \nORGANISM    hsa  Homo sapiens (human)\nPATHWAY
# PDB: 3RNJ 4JS0 2YKT 1Y2O 1WDZ\nAASEQ       552\n            MSLSRS
# HGDGSARTLAGR\nNTSEQ
def parsegetpro(propage):
    spename, speseq = None, []
    apipage = propage.read()
    apipagelist = apipage.split("///")
    for line in apipagelist:
        regname = r"(?<=\nORGANISM).*?(?=\nPATHWAY)"
        regseq = r"(?<=\nAASEQ).*?(?=\nNTSEQ)"
        namecomp = re.compile(regname)
        seqcomp = re.compile(regseq)
        name = re.findall(namecomp, line)
        seq = re.findall(seqcomp, line)



def getprotein(proid):
    # http://rest.kegg.jp/get/hsa:10458+ece:Z5100+pon:100172290
    url = "http://rest.kegg.jp/get/" + proid + "/aaseq"
    if (python_version == 3):
        page = request.urlopen(url)
    else:
        page = urllib2.urlopen(url)
    return page



from setuptools import setup, find_packages
from physpetool.version import version

physpe_version = version

try:
    LONG_DESCRIPTION = open("README.rst", "rb").read().decode("utf-8")
except:
    long_descr = "One command auto reconstruct phylogenetic tree"
setup(
    name="PhySpeTree",
    packages=find_packages(),
    entry_points={
        "console_scripts": ['PhySpeTree = physpetool.physpe.physpe:main']
    },
    version=physpe_version,
    description="One command line auto reconstruct phylogenetic tree.",
    long_description=LONG_DESCRIPTION,
    author="Yang Fang",
    author_email="yangfangscu@gmail.com",
    package_data={'': ['*.rst'], 'physpetool': ['database/*.db', 'softwares/*','softwares/lib/*', 'database/*.txt']},
    url="https://github.com/yangfangs/physpetools",
)



import os

read_dir = "/home/yangfang/PhySpeTree/revision/HCPDB/hcp_append/"
write_dir = "/home/yangfang/PhySpeTree/revision/HCPDB/hcp_append_trans/"

all_files = list(filter(lambda f: not f.startswith('.'), os.listdir(read_dir)))

for line in all_files:
    oldname = read_dir + line
    newname = read_dir + line[1:]
    os.rename(oldname, newname)


# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The operate sqlite3 DB class

"""

import sqlite3


class operate_db():
    def __init__(self, db_path):
        """connect a  DB with file path"""
        self.conn = sqlite3.connect(db_path)
        self.conn.text_factory = str
        self.cur = self.conn.cursor()

    def create(self, table, colname_list):
        """create a table with col name"""
        text = ' TEXT, '.join(colname_list)
        print (text)
        createlist = ["create table if not exists %s" % table, "(id integer primary key autoincrement, NAME TEXT, ",
                      text, " TEXT)"]
        createsql = ''.join(createlist)
        try:
            self.cur.execute(createsql)
            self.done()
        except sqlite3.Error as e:
            print(e)
            self.conn.rollback()

    def insert(self, table, name_list):
        """insert name to table"""
        insert_name = []
        for line in name_list:
            insert_name.append((line,))
        try:
            # self.cur.executemany("INSERT INTO {0}(NAME) VALUES ({1})".format(table, insert_name))
            sql_insert = "INSERT INTO %s (NAME) VALUES (?)" % table
            self.cur.executemany(sql_insert, insert_name)
            self.done()
        except sqlite3.Error as e:
            print(e)
            self.conn.rollback()

    def update(self, table, col_name, name_flag, value):
        """update table example:update('table_one','K02865','hsa','has:2525525')"""
        sql_update = "UPDATE {0} SET {1}=? WHERE NAME=?".format(table, col_name)
        try:
            self.cur.execute(sql_update, (value, name_flag))
            self.done()
        except sqlite3.Error as e:
            print (e)
            self.conn.rollback()

    def display_row(self, table, abb_name):
        """display the row by abb name"""
        sql_disply = "SELECT * FROM {0} WHERE NAME='{1}'".format(table, abb_name)
        try:
            self.cur.execute(sql_disply)
            result = self.cur.fetchall()
            self.done()
            print (result)
        except sqlite3.Error as e:
            print (e)
            self.conn.rollback()

    def dispay_col(self, table, col_name):
        """get the table col data by col name"""
        sql_select = "SELECT {0} FROM {1}".format(col_name, table)
        try:
            self.cur.execute(sql_select)
            col_data = self.cur.fetchall
            self.done()
            return col_data
        except sqlite3.Error as e:
            print (e)
            self.conn.rollback()

    def drop(self, table):
        """drop the table"""
        try:
            self.cur.execute("DROP TABLE IF EXISTS %s" % table)
            self.done()
        except sqlite3.Error as e:
            print (e)
            self.conn.rollback()

    def done(self):
        """commit"""
        self.conn.commit()

    def close(self):
        """close DB"""
        self.conn.close()



def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass

    return False


# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

import os
import subprocess
import re
from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath

loggtrimal = getLogging('trimal')


# trimal -in concatenate.fasta -out output2 -gt 1 -phylip3.2

def dotrimal(indata, trimalpara):
    """
    do trimal after muslce and concatenate
    :param indata: a fasta file input to do trimal
    :param outdata: append name after
    :return: a file path of trimal result
    """
    # Deal with outdata name
    trimalparas = trimalpara.lstrip()
    trimalpath = getlocalpath()
    out_path = os.path.dirname(indata)
    trimal_name = "trimal.phy"
    trimal_data = os.path.join(out_path, trimal_name)

    cmd = trimalpath + "/trimal " + " -in " + indata + " -out " + trimal_data + " " + trimalparas + " -phylip"

    subprocess.call(cmd, shell=True)
    # support fasttree software
    trimal_data2 = trimal_data.replace('.phy', '')
    cmd2 = trimalpath + "/trimal " + " -in " + indata + " -out " + trimal_data2 + " " + trimalparas + " -fasta"
    subprocess.call(cmd2, shell=True)

    loggtrimal.info('Select conserved blocks by trimal was completed')
    loggtrimal.debug('trimal path:{0}'.format(trimal_data))
    return trimal_data
